﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using ThinController.Interfaces;

namespace ThinController.Classes
{
    public class DefaultViewModelFactory: IViewModelFactory
    {
        public TViewModel GetViewModel<TController, TViewModel>(TController controller)
        {
            var viewModels = GetSystemTypes(controller, typeof (TViewModel));
            var modelBuilders = GetSystemTypes(controller, typeof(IViewModelBuilder<TController, TViewModel>));

            var typeName = typeof(TViewModel).Name;
            var model = (from t in viewModels
                             where typeof(TViewModel).IsAssignableFrom(t) && t.Name.Equals(typeName)
                             let strategy = (TViewModel)Activator.CreateInstance(t)
                             select strategy).FirstOrDefault();

            var modelBuilder = (from t in modelBuilders
                             where typeof(IViewModelBuilder<TController, TViewModel>).IsAssignableFrom(t)
                             let strategy = (IViewModelBuilder<TController, TViewModel>)Activator.CreateInstance(t)
                             select strategy).FirstOrDefault();

            // Redirect and assist developers in adding their own modelbuilder/viewmodel
            if (modelBuilder == null)
                throw new Exception(
                    String.Format(
                        "Could not find a ModelBuilder with a {0} Controller/{1} ViewModel pairing. Please create one.",
                        typeof (TController).Name, typeof (TViewModel).Name));
            
            return modelBuilder.Build(controller, model);
        }

        private IEnumerable<Type> GetSystemTypes<TController>(TController controller, Type type)
        {
            // First, get the types of the executing assembly
            var currentAssemblyTypes = Assembly
                .GetExecutingAssembly()
                .GetTypes()
                .Where(e => type.IsAssignableFrom(e))
                .ToList();

            // Then the calling assembly
            currentAssemblyTypes.AddRange(
                Assembly
                    .GetCallingAssembly()
                    .GetTypes()
                    .Where(e => type.IsAssignableFrom(e))
                    .ToList()
                );

            return currentAssemblyTypes.Distinct();
        }
    }
}
